@@ -1,39 +1,57 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
-""" Nipype interfaces to support anatomical workflow """
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
+"""Nipype interfaces to support anatomical workflow."""
 import os.path as op
-import numpy as np
-import nibabel as nb
-from math import sqrt
-import scipy.ndimage as nd
 from builtins import zip
 
-from nipype.utils.filemanip import fname_presuffix
-from nipype.interfaces.base import (
-    traits,
-    TraitedSpec,
-    File,
-    isdefined,
-    InputMultiPath,
-    BaseInterfaceInputSpec,
-    SimpleInterface,
-)
-
-from ..utils.misc import _flatten_dict
-from ..qc.anatomical import (
-    snr,
-    snr_dietrich,
-    cnr,
-    fber,
-    efc,
+import nibabel as nb
+import numpy as np
+import scipy.ndimage as nd
+from mriqc.qc.anatomical import (
     art_qi1,
     art_qi2,
-    volume_fraction,
-    rpve,
-    summary_stats,
     cjv,
+    cnr,
+    efc,
+    fber,
+    rpve,
+    snr,
+    snr_dietrich,
+    summary_stats,
+    volume_fraction,
     wm2max,
 )
+from mriqc.utils.misc import _flatten_dict
+from nipype.interfaces.base import (
+    BaseInterfaceInputSpec,
+    File,
+    InputMultiPath,
+    SimpleInterface,
+    TraitedSpec,
+    isdefined,
+    traits,
+)
+from nipype.utils.filemanip import fname_presuffix
 
 
 class StructuralQCInputSpec(BaseInterfaceInputSpec):
@@ -46,13 +64,16 @@ class StructuralQCInputSpec(BaseInterfaceInputSpec):
     rot_msk = File(exists=True, mandatory=True, desc="rotation mask")
     artifact_msk = File(exists=True, mandatory=True, desc="air mask")
     in_pvms = InputMultiPath(
-        File(exists=True), mandatory=True, desc="partial volume maps from FSL FAST"
+        File(exists=True),
+        mandatory=True,
+        desc="partial volume maps from FSL FAST",
     )
     in_tpms = InputMultiPath(File(), desc="tissue probability maps from FSL FAST")
     mni_tpms = InputMultiPath(File(), desc="tissue probability maps from FSL FAST")
     in_fwhm = traits.List(
         traits.Float, mandatory=True, desc="smoothness estimated with AFNI"
     )
+    human = traits.Bool(True, usedefault=True, desc="human workflow")
 
 
 class StructuralQCOutputSpec(TraitedSpec):
@@ -88,26 +109,62 @@ class StructuralQC(SimpleInterface):
 
     def _run_interface(self, runtime):  # pylint: disable=R0914,E1101
         imnii = nb.load(self.inputs.in_noinu)
-        erode = np.all(np.array(imnii.header.get_zooms()[:3], dtype=np.float32) < 1.9)
+        erode = (
+            np.all(np.array(imnii.header.get_zooms()[:3], dtype=np.float32) < 1.9)
+            if self.inputs.human
+            else False
+        )
 
         # Load image corrected for INU
-        inudata = np.nan_to_num(imnii.get_data())
+        inudata = np.nan_to_num(imnii.get_fdata())
         inudata[inudata < 0] = 0
 
+        if np.all(inudata < 1e-3):
+            raise RuntimeError(
+                "Input inhomogeneity-corrected data seem empty. "
+                "MRIQC failed to process this dataset."
+            )
+
         # Load binary segmentation from FSL FAST
         segnii = nb.load(self.inputs.in_segm)
-        segdata = segnii.get_data().astype(np.uint8)
+        segdata = np.asanyarray(segnii.dataobj).astype(np.uint8)
+
+        if np.sum(segdata > 0) < 1e3:
+            raise RuntimeError(
+                "Input segmentation data is likely corrupt. "
+                "MRIQC failed to process this dataset."
+            )
 
         # Load air, artifacts and head masks
-        airdata = nb.load(self.inputs.air_msk).get_data().astype(np.uint8)
-        artdata = nb.load(self.inputs.artifact_msk).get_data().astype(np.uint8)
-        headdata = nb.load(self.inputs.head_msk).get_data().astype(np.uint8)
-        rotdata = nb.load(self.inputs.rot_msk).get_data().astype(np.uint8)
+        airdata = np.asanyarray(nb.load(self.inputs.air_msk).dataobj).astype(np.uint8)
+        if np.sum(airdata > 0) < 100:
+            raise RuntimeError(
+                "Detected less than 100 voxels belonging to the air mask. "
+                "MRIQC failed to process this dataset."
+            )
+
+        artdata = np.asanyarray(nb.load(self.inputs.artifact_msk).dataobj).astype(
+            np.uint8
+        )
+
+        headdata = np.asanyarray(nb.load(self.inputs.head_msk).dataobj).astype(np.uint8)
+        if np.sum(headdata > 0) < 100:
+            raise RuntimeError(
+                "Detected less than 100 voxels belonging to the head mask. "
+                "MRIQC failed to process this dataset."
+            )
+
+        rotdata = np.asanyarray(nb.load(self.inputs.rot_msk).dataobj).astype(np.uint8)
 
         # Load Partial Volume Maps (pvms) from FSL FAST
         pvmdata = []
         for fname in self.inputs.in_pvms:
-            pvmdata.append(nb.load(fname).get_data().astype(np.float32))
+            pvmdata.append(nb.load(fname).get_fdata(dtype="float32"))
+            if np.sum(pvmdata[-1] > 1e-4) < 10:
+                raise RuntimeError(
+                    "Detected less than 10 voxels belonging to one tissue prob. map. "
+                    "MRIQC failed to process this dataset."
+                )
 
         # Summary stats
         stats = summary_stats(inudata, pvmdata, airdata, erode=erode)
@@ -118,7 +175,11 @@ class StructuralQC(SimpleInterface):
         self._results["snr"] = {}
         for tlabel in ["csf", "wm", "gm"]:
             snrvals.append(
-                snr(stats[tlabel]["median"], stats[tlabel]["stdv"], stats[tlabel]["n"])
+                snr(
+                    stats[tlabel]["median"],
+                    stats[tlabel]["stdv"],
+                    stats[tlabel]["n"],
+                )
             )
             self._results["snr"][tlabel] = snrvals[-1]
         self._results["snr"]["total"] = float(np.mean(snrvals))
@@ -136,7 +197,9 @@ class StructuralQC(SimpleInterface):
         self._results["cnr"] = cnr(
             stats["wm"]["median"],
             stats["gm"]["median"],
-            sqrt(sum(stats[k]["stdv"] ** 2 for k in ["bg", "gm", "wm"])),
+            stats["bg"]["stdv"],
+            stats["wm"]["stdv"],
+            stats["gm"]["stdv"],
         )
 
         # FBER
@@ -198,7 +261,7 @@ class StructuralQC(SimpleInterface):
             pass
 
         # Bias
-        bias = nb.load(self.inputs.in_bias).get_data()[segdata > 0]
+        bias = nb.load(self.inputs.in_bias).get_fdata()[segdata > 0]
         self._results["inu"] = {
             "range": float(
                 np.abs(np.percentile(bias, 95.0) - np.percentile(bias, 5.0))
@@ -206,8 +269,8 @@ class StructuralQC(SimpleInterface):
             "med": float(np.median(bias)),
         }  # pylint: disable=E1101
 
-        mni_tpms = [nb.load(tpm).get_data() for tpm in self.inputs.mni_tpms]
-        in_tpms = [nb.load(tpm).get_data() for tpm in self.inputs.in_pvms]
+        mni_tpms = [nb.load(tpm).get_fdata() for tpm in self.inputs.mni_tpms]
+        in_tpms = [nb.load(tpm).get_fdata() for tpm in self.inputs.in_pvms]
         overlap = fuzzy_jaccard(in_tpms, mni_tpms)
         self._results["tpm_overlap"] = {
             "csf": overlap[0],
@@ -225,7 +288,9 @@ class ArtifactMaskInputSpec(BaseInterfaceInputSpec):
     head_mask = File(exists=True, mandatory=True, desc="head mask")
     rot_mask = File(exists=True, desc="a rotation mask")
     nasion_post_mask = File(
-        exists=True, mandatory=True, desc="nasion to posterior of cerebellum mask"
+        exists=True,
+        mandatory=True,
+        desc="nasion to posterior of cerebellum mask",
     )
 
 
@@ -245,13 +310,13 @@ class ArtifactMask(SimpleInterface):
 
     def _run_interface(self, runtime):
         imnii = nb.load(self.inputs.in_file)
-        imdata = np.nan_to_num(imnii.get_data().astype(np.float32))
+        imdata = np.nan_to_num(imnii.get_fdata().astype(np.float32))
 
         # Remove negative values
         imdata[imdata < 0] = 0
 
-        hmdata = nb.load(self.inputs.head_mask).get_data()
-        npdata = nb.load(self.inputs.nasion_post_mask).get_data()
+        hmdata = np.asanyarray(nb.load(self.inputs.head_mask).dataobj)
+        npdata = np.asanyarray(nb.load(self.inputs.nasion_post_mask).dataobj)
 
         # Invert head mask
         airdata = np.ones_like(hmdata, dtype=np.uint8)
@@ -267,7 +332,7 @@ class ArtifactMask(SimpleInterface):
 
         # Apply rotation mask (if supplied)
         if isdefined(self.inputs.rot_mask):
-            rotmskdata = nb.load(self.inputs.rot_mask).get_data()
+            rotmskdata = np.asanyarray(nb.load(self.inputs.rot_mask).dataobj)
             airdata[rotmskdata == 1] = 0
 
         # Run the artifact detection
@@ -318,8 +383,8 @@ class ComputeQI2(SimpleInterface):
     output_spec = ComputeQI2OutputSpec
 
     def _run_interface(self, runtime):
-        imdata = nb.load(self.inputs.in_file).get_data()
-        airdata = nb.load(self.inputs.air_msk).get_data()
+        imdata = nb.load(self.inputs.in_file).get_fdata()
+        airdata = nb.load(self.inputs.air_msk).get_fdata()
         qi2, out_file = art_qi2(imdata, airdata)
         self._results["qi2"] = qi2
         self._results["out_file"] = out_file
@@ -332,6 +397,7 @@ class HarmonizeInputSpec(BaseInterfaceInputSpec):
     )
     wm_mask = File(exists=True, mandatory=True, desc="white-matter mask")
     erodemsk = traits.Bool(True, usedefault=True, desc="erode mask")
+    thresh = traits.Float(0.9, usedefault=True, desc="WM probability threshold")
 
 
 class HarmonizeOutputSpec(TraitedSpec):
@@ -349,7 +415,7 @@ class Harmonize(SimpleInterface):
     def _run_interface(self, runtime):
 
         in_file = nb.load(self.inputs.in_file)
-        wm_mask = nb.load(self.inputs.wm_mask).get_data()
+        wm_mask = nb.load(self.inputs.wm_mask).get_fdata()
         wm_mask[wm_mask < 0.9] = 0
         wm_mask[wm_mask > 0] = 1
         wm_mask = wm_mask.astype(np.uint8)
@@ -360,7 +426,7 @@ class Harmonize(SimpleInterface):
             # Perform an opening operation on the background data.
             wm_mask = nd.binary_erosion(wm_mask, structure=struc).astype(np.uint8)
 
-        data = in_file.get_data()
+        data = in_file.get_fdata()
         data *= 1000.0 / np.median(data[wm_mask > 0])
 
         out_file = fname_presuffix(
@@ -391,7 +457,7 @@ class RotationMask(SimpleInterface):
 
     def _run_interface(self, runtime):
         in_file = nb.load(self.inputs.in_file)
-        data = in_file.get_data()
+        data = in_file.get_fdata()
         mask = data <= 0
 
         # Pad one pixel to control behavior on borders of binary_opening
