@@ -1,8 +1,28 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
-"""Helper functions for the workflows"""
-from distutils.version import StrictVersion
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
+"""Helper functions for the workflows."""
 from builtins import range
+from distutils.version import StrictVersion
 
 
 def _tofloat(inlist):
@@ -12,50 +32,35 @@ def _tofloat(inlist):
         return float(inlist)
 
 
-def fmri_getidx(in_file, start_idx, stop_idx):
-    """Heuristics to set the start and stop indices of fMRI series"""
-    from nibabel import load
-    from nipype.interfaces.base import isdefined
-    nvols = load(in_file).shape[3]
-    max_idx = nvols - 1
-
-    if start_idx is None or not isdefined(start_idx) or start_idx < 0 or start_idx > max_idx:
-        start_idx = 0
-
-    if (
-        stop_idx is None
-        or not isdefined(stop_idx)
-        or max_idx < stop_idx < start_idx
-    ):
-        stop_idx = max_idx
-    return start_idx, stop_idx
-
-
 def fwhm_dict(fwhm):
     """Convert a list of FWHM into a dictionary"""
     fwhm = [float(f) for f in fwhm]
-    return {'fwhm_x': fwhm[0], 'fwhm_y': fwhm[1],
-            'fwhm_z': fwhm[2], 'fwhm_avg': fwhm[3]}
+    return {
+        "fwhm_x": fwhm[0],
+        "fwhm_y": fwhm[1],
+        "fwhm_z": fwhm[2],
+        "fwhm_avg": fwhm[3],
+    }
 
 
 def thresh_image(in_file, thres=0.5, out_file=None):
     """Thresholds an image"""
     import os.path as op
+
     import nibabel as nb
 
     if out_file is None:
         fname, ext = op.splitext(op.basename(in_file))
-        if ext == '.gz':
+        if ext == ".gz":
             fname, ext2 = op.splitext(fname)
             ext = ext2 + ext
-        out_file = op.abspath('{}_thresh{}'.format(fname, ext))
+        out_file = op.abspath("{}_thresh{}".format(fname, ext))
 
     im = nb.load(in_file)
     data = im.get_data()
     data[data < thres] = 0
     data[data > 0] = 1
-    nb.Nifti1Image(
-        data, im.affine, im.header).to_filename(out_file)
+    nb.Nifti1Image(data, im.affine, im.header).to_filename(out_file)
     return out_file
 
 
@@ -86,19 +91,20 @@ def spectrum_mask(size):
     return ftmask
 
 
-def slice_wise_fft(in_file, ftmask=None, spike_thres=3., out_prefix=None):
+def slice_wise_fft(in_file, ftmask=None, spike_thres=3.0, out_prefix=None):
     """Search for spikes in slices using the 2D FFT"""
     import os.path as op
-    import numpy as np
+
     import nibabel as nb
+    import numpy as np
     from mriqc.workflows.utils import spectrum_mask
+    from scipy.ndimage import binary_erosion, generate_binary_structure
     from scipy.ndimage.filters import median_filter
-    from scipy.ndimage import generate_binary_structure, binary_erosion
     from statsmodels.robust.scale import mad
 
     if out_prefix is None:
         fname, ext = op.splitext(op.basename(in_file))
-        if ext == '.gz':
+        if ext == ".gz":
             fname, _ = op.splitext(fname)
         out_prefix = op.abspath(fname)
 
@@ -113,8 +119,14 @@ def slice_wise_fft(in_file, ftmask=None, spike_thres=3., out_prefix=None):
         fft_slices = []
         for z in range(func_frame.shape[2]):
             sl = func_frame[..., z]
-            fftsl = median_filter(np.real(np.fft.fft2(sl)).astype(np.float32),
-                                  size=(5, 5), mode='constant') * ftmask
+            fftsl = (
+                median_filter(
+                    np.real(np.fft.fft2(sl)).astype(np.float32),
+                    size=(5, 5),
+                    mode="constant",
+                )
+                * ftmask
+            )
             fft_slices.append(fftsl)
         fft_data.append(np.stack(fft_slices, axis=-1))
 
@@ -129,7 +141,7 @@ def slice_wise_fft(in_file, ftmask=None, spike_thres=3., out_prefix=None):
     fft_zscored[idxs] /= sigma[idxs]
 
     # save fft z-scored
-    out_fft = op.abspath(out_prefix + '_zsfft.nii.gz')
+    out_fft = op.abspath(out_prefix + "_zsfft.nii.gz")
     nii = nb.Nifti1Image(fft_zscored.astype(np.float32), np.eye(4), None)
     nii.to_filename(out_fft)
 
@@ -154,18 +166,18 @@ def slice_wise_fft(in_file, ftmask=None, spike_thres=3., out_prefix=None):
             if sl.sum() > 10:
                 spikes_list.append((t, z))
 
-    out_spikes = op.abspath(out_prefix + '_spikes.tsv')
-    np.savetxt(out_spikes, spikes_list, fmt=b'%d', delimiter=b'\t', header='TR\tZ')
+    out_spikes = op.abspath(out_prefix + "_spikes.tsv")
+    np.savetxt(out_spikes, spikes_list, fmt=b"%d", delimiter=b"\t", header="TR\tZ")
 
     return len(spikes_list), out_spikes, out_fft
 
 
 def get_fwhmx():
-    from nipype.interfaces.afni import Info, FWHMx
-    fwhm_args = {"combine": True,
-                 "detrend": True}
-    afni_version = StrictVersion('%s.%s.%s' % Info.version())
+    from nipype.interfaces.afni import FWHMx, Info
+
+    fwhm_args = {"combine": True, "detrend": True}
+    afni_version = StrictVersion("%s.%s.%s" % Info.version())
     if afni_version >= StrictVersion("2017.2.3"):
-        fwhm_args['args'] = '-ShowMeClassicFWHM'
+        fwhm_args["args"] = "-ShowMeClassicFWHM"
     fwhm_interface = FWHMx(**fwhm_args)
     return fwhm_interface
